@@ -1,14 +1,19 @@
 "alpha" <- 
-    function(x,keys=NULL,cumulative=FALSE,title=NULL,max=10,na.rm=TRUE,check.keys=FALSE,n.iter=1,delete=TRUE,use="pairwise",warnings=TRUE,n.obs=NULL) {  #find coefficient alpha given a data frame or a matrix
+    function(x,keys=NULL,cumulative=FALSE,title=NULL,max=10,na.rm=TRUE,check.keys=FALSE,n.iter=1,delete=TRUE,use="pairwise",warnings=TRUE,n.obs=NULL,impute=NULL) {  #find coefficient alpha given a data frame or a matrix
     
-    alpha.1 <- function(C,R) {
+ alpha.1 <- function(C,R) {
     n <- dim(C)[2]
     alpha.raw <- (1- tr(C)/sum(C))*(n/(n-1))
     sumR <- sum(R)
-    alpha.std <-  (1- n/sumR)*(n/(n-1))
+    alpha.std <-  (1- n/sumR)*(n/(n-1))  
     smc.R <- smc(R)
     G6 <- (1- (n-sum(smc.R))/sumR)
     av.r <- (sumR-n)/(n*(n-1))
+    R.adj <- R[lower.tri(R)]
+   # diag(R.adj) <- NA
+   # var.r  <- var(as.vector(R.adj),na.rm=TRUE)
+   var.r <- var(R.adj,na.rm=TRUE)  #
+   med.r <- median(R.adj,na.rm=TRUE)  #added 4/22/18
     mod1 <- matrix(av.r,n,n)
     Res1 <- R - mod1
     GF1 =  1- sum(Res1^2)/sum(R^2)
@@ -18,7 +23,7 @@
     sn <- n*av.r/(1-av.r)
    # Q = (2 * n^2/((n-1)^2*(sum(C)^3))) * (sum(C) * (tr(C^2) + (tr(C))^2) - 2*(tr(C) * sum(C^2))) #corrected 1/15/16 
     Q = (2 * n^2/((n - 1)^2 * (sum(C)^3))) * (sum(C) * (tr(C%*%C) +  (tr(C))^2) - 2 * (tr(C) * sum(C%*%C)))   #correction from Tamaki Hattori
-    result <- list(raw=alpha.raw,std=alpha.std,G6=G6,av.r=av.r,sn=sn,Q=Q,GF1,GF1.off)
+    result <- list(raw=alpha.raw,std=alpha.std,G6=G6,av.r=av.r,sn=sn,Q=Q,GF1,GF1.off,var.r = var.r,med.r=med.r)
     return(result)
     }
     
@@ -26,17 +31,29 @@
     cl <- match.call()
     if(!is.matrix(x) && !is.data.frame(x)) stop('Data must either be a data frame or a matrix')
     if(class(x)[1] != "data.frame") x <- fix.dplyr(x)    #to get around a problem created by dplyr
-    if(!is.null(keys)){#two 3 cases  1 it is a list, 2 is a vector of character, 3 it is keys matrix  4 it is a list of items to  reverse
+    if(!is.null(keys)){# 3 cases  1 it is a list, 2 is a vector of character, 3 it is keys matrix  4 it is a list of items to  reverse
     if( is.list(keys)) { select <- sub("-","",unlist(keys))   #added 9/26/16 to speed up scoring one scale from many
-      x <- x[select] 
-      keys <- make.keys(x,keys)}}
+      	x <- x[,select] 
+      	keys <- make.keys(x,keys)} else {
+      	temp <- rep(1,ncol(x))
+      	temp[(colnames(x) %in% keys)] <- -1
+      	keys <- temp
+      }
+      }
       
     nvar <- dim(x)[2]
     nsub <- dim(x)[1]
     scores <- NULL
     response.freq <- NULL
-   
+    raw <- FALSE
     if (!isCorrelation(x))  { #find the correlations if we are given  raw data
+      raw <- TRUE
+       if(!is.null(impute) ) {
+       		if(impute=="median") {item.impute <- apply(x,2,median,na.rm=na.rm)} else {item.impute <- apply(x,2,mean,na.rm=na.rm) } #column values
+  #     		apply(x,1,function(x) { x[is.na(x)]  <- item.impute[]}) }  #replace row wise
+                 
+            for(i in 1:nsub) {for (j in 1:nvar) {x[i,is.na(x[i,j])] <- item.impute[j] }   }  
+            }
        item.var <- apply(x,2,sd,na.rm=na.rm)
        bad <- which((item.var <= 0)|is.na(item.var))
        if((length(bad) > 0) && delete) {
@@ -48,31 +65,44 @@
          C <- cov(x,use=use)} else {C <- x}
         
         if(is.null(colnames(x)))  colnames(x) <- paste0("V",1:nvar)
-       
+         #flip items if needed and wanted
          #if(check.keys && is.null(keys)) {
             p1 <- principal(x,scores=FALSE)
+
                if(any(p1$loadings < 0)) {if (check.keys) {if(warnings) warning("Some items were negatively correlated with total scale and were automatically reversed.\n This is indicated by a negative sign for the variable name.") 
-                    keys <- 1- 2* (p1$loadings < 0 ) } else {
+                    keys <- 1- 2* (p1$loadings < 0)
+                      } else {
                        if(is.null(keys) && warnings ) {warning("Some items were negatively correlated with the total scale and probably \nshould be reversed.  \nTo do this, run the function again with the 'check.keys=TRUE' option")
                        if(warnings) cat("Some items (",rownames(p1$loadings)[(p1$loadings < 0)],") were negatively correlated with the total scale and \nprobably should be reversed.  \nTo do this, run the function again with the 'check.keys=TRUE' option")
-                        }} 
+                        keys <- rep(1,nvar)
+                         } 
+                   
+                    }
             }  #keys is now a vector of 1s and -1s
+           #names(keys) <- colnames(x)
            # }
             
-         if(is.null(keys)) {keys <- rep(1,nvar)} else {  
+         if(is.null(keys)) {keys <- rep(1,nvar)
+              
+                 names(keys) <- colnames(x)} else {  
          			keys<- as.vector(keys) 
+         	        names(keys) <- colnames(x)
          			if(length(keys) < nvar) {temp <- keys  #this is the option of keying just the reversals
          			                         keys <- rep(1,nvar)
          			                         names(keys) <- colnames(x)
-         			                         keys[temp] <- -1}
+         			                         keys[temp] <- -1
+         			                      }
+         			                         
          			                   } 
+         			           
+         			           
         			 key.d <- diag(keys)
                      C <- key.d %*% C %*% key.d
                      signkey <- strtrim(keys,1)
             		 signkey[signkey=="1"] <- ""
                      colnames(x) <- paste(colnames(x),signkey,sep="")
                      
-      if (nsub !=nvar)  {   #raw data      
+      if (raw)  {   #raw data      
          	if(any(keys < 0 )) { 
          	     min.item <- min(x,na.rm=na.rm)
                  max.item <- max(x,na.rm=na.rm)
@@ -102,15 +132,15 @@
                             } 
          } else {drop.item[[1]] <- drop.item[[2]] <- c(rep(R[1,2],2),smc(R)[1],R[1,2],NA,NA,NA,NA)  #added the extra 2 NA June 18, 2017
        }
-        by.item <- data.frame(matrix(unlist(drop.item),ncol=8,byrow=TRUE)) 
+        by.item <- data.frame(matrix(unlist(drop.item),ncol=10,byrow=TRUE)) 
         
                   #allows us to specify the number of subjects for correlation matrices
         if(max(nsub,n.obs) > nvar) {by.item[6] <- sqrt(by.item[6]/(max(nsub,n.obs)) )
          by.item <- by.item[-c(7:8)]
-         colnames(by.item) <- c("raw_alpha","std.alpha","G6(smc)","average_r","S/N","alpha se") } else {
+         colnames(by.item) <- c("raw_alpha","std.alpha","G6(smc)","average_r","S/N","alpha se","var.r","med.r") } else {
          
              by.item <- by.item[-c(6:8)]
-             colnames(by.item) <- c("raw_alpha","std.alpha","G6(smc)","average_r","S/N") }
+             colnames(by.item) <- c("raw_alpha","std.alpha","G6(smc)","average_r","S/N","var.r","med.r") }
         rownames(by.item) <- colnames(x)
         
         Vt <- sum(R)
@@ -133,28 +163,32 @@
      #  
         item.means <- colMeans(x, na.rm=na.rm )
         item.sd <-  apply(x,2,sd,na.rm=na.rm)
-        if(nsub > nvar) {
+        if(raw) {
          Unidim <- alpha.total[7]
+         var.r <- alpha.total[[9]]
         	 Fit.off <- alpha.total[8]
             ase = sqrt(alpha.total$Q/nsub)
-        	alpha.total <- data.frame(alpha.total[1:5],ase=ase,mean=mean.t,sd=sdev)
-           colnames(alpha.total) <- c("raw_alpha","std.alpha","G6(smc)","average_r","S/N","ase","mean","sd")
+        	#alpha.total <- data.frame(alpha.total[1:5],ase=ase,mean=mean.t,sd=sdev)
+          # colnames(alpha.total) <- c("raw_alpha","std.alpha","G6(smc)","average_r","S/N","ase","mean","sd")
         	
         	  
-        	 alpha.total <- data.frame(alpha.total[1:5],ase=ase,mean=mean.t,sd=sdev)
-        	colnames(alpha.total) <- c("raw_alpha","std.alpha","G6(smc)","average_r","S/N","ase","mean","sd")
+        	 alpha.total <- data.frame(alpha.total[1:5],ase=ase,mean=mean.t,sd=sdev,med.r =alpha.total[10])
+        	colnames(alpha.total) <- c("raw_alpha","std.alpha","G6(smc)","average_r","S/N","ase","mean","sd","median_r")
         	rownames(alpha.total) <- ""
         	stats <- data.frame(n=t.valid,raw.r=t(raw.r),std.r =item.r,r.cor = item.rc,r.drop = r.drop,mean=item.means,sd=item.sd)
         	} else {
         	if(is.null(n.obs)) {
         	       Unidim <- alpha.total[7]
         	       Fit.off <- alpha.total[8]
-        	      alpha.total <- data.frame(alpha.total[1:5])  #fixed 27/7/14 
-        	        colnames(alpha.total) <- c("raw_alpha","std.alpha","G6(smc)" ,"average_r","S/N") } else {
+        	       var.r <- alpha.total[9] 
+        	       med.r <- alpha.total[10]
+        	      alpha.total <- data.frame(alpha.total[c(1:5,10)])  #fixed 27/7/14 
+        	        colnames(alpha.total) <- c("raw_alpha","std.alpha","G6(smc)" ,"average_r","S/N","median_r") } else {
         	        Unidim <- alpha.total[7]
         	 		Fit.off <- alpha.total[8] 
-        	        alpha.total <- data.frame(alpha.total[1:5],ase=sqrt(alpha.total$Q/n.obs))
-        	        colnames(alpha.total) <- c("raw_alpha","std.alpha","G6(smc)" ,"average_r","S/N","ase")}
+        	 		        	       var.r <- alpha.total[9] 
+        	        alpha.total <- data.frame(alpha.total[1:5],ase=sqrt(alpha.total$Q/n.obs),alpha.total[10])
+        	        colnames(alpha.total) <- c("raw_alpha","std.alpha","G6(smc)" ,"average_r","S/N","ase","median_r")}
         	        rownames(alpha.total) <- "" 
         	        
 
@@ -170,7 +204,7 @@
        	#end of unidimensionality statistics
        	if(n.iter > 1) {#do a bootstrap confidence interval for alpha
    #    	 if(!require(parallel)) {message("The parallel package needs to be installed to run mclapply")}
-        if(nsub == nvar) {message("bootstrapped confidence intervals require raw data") 
+        if(!raw) {message("bootstrapped confidence intervals require raw data") 
                           boot <- NULL
                           boot.ci <- NULL } else {
        	 boot <- vector("list",n.iter)
@@ -185,14 +219,14 @@
        	 alpha.1(C,R)
        	 })  #end of mclapply 
        
-       	  boot <- matrix(unlist(boot),ncol=8,byrow=TRUE)
-       	  colnames(boot) <- c("raw_alpha","std.alpha","G6(smc)","average_r","s/n","ase","Unidim","Goodfit")
+       	  boot <- matrix(unlist(boot),ncol=10,byrow=TRUE)
+       	  colnames(boot) <- c("raw_alpha","std.alpha","G6(smc)","average_r","s/n","ase","Unidim","Goodfit","var.r","median.r")
        	  boot.ci <- quantile(boot[,1],c(.025,.5,.975))
        	}} else {boot=NULL
        	         boot.ci <- NULL}
        	names(Unidim) <- "Unidim"
        	names(Fit.off) <- "Fit.off" 
-        result <- list(total=alpha.total,alpha.drop=by.item,item.stats=stats,response.freq=response.freq,keys=keys,scores = total,nvar=nvar,boot.ci=boot.ci,boot=boot,Unidim=Unidim,Fit=Fit.off,call=cl,title=title)
+        result <- list(total=alpha.total,alpha.drop=by.item,item.stats=stats,response.freq=response.freq,keys=keys,scores = total,nvar=nvar,boot.ci=boot.ci,boot=boot,Unidim=Unidim,var.r=var.r,Fit=Fit.off,call=cl,title=title)
         class(result) <- c("psych","alpha")
         return(result) 
     }
@@ -223,4 +257,17 @@ cn <- c("data.frame",cn.not)
 class(object) <- cn
       } 
 invisible(object) 
-}
\ No newline at end of file
+}
+
+#apply the Duhacheck and Iacobucci estimates
+#compare with Feldt's estimate
+"alpha.ci" <- function(alpha,n.obs,n.var=NULL,p.val=.05,digits=2) {
+#  Q = (2 * n^2/((n - 1)^2 * (sum(C)^3))) * (sum(C) * (tr(C%*%C) +  (tr(C))^2) - 2 * (tr(C) * sum(C%*%C)))   #correction from Tamaki Hattori
+ CI.high <- 1- (1-alpha)* qf(p.val/2,n.obs-1,Inf)
+ CI.low <- 1-  (1-alpha)* qf(1-p.val/2,n.obs-1,Inf)
+if(!is.null(n.var)) {r.bar <- alpha/(n.var - alpha*(n.var-1)) } else {r.bar=NA}
+result <- list(lower.ci =CI.low,alpha=alpha,upper.ci=CI.high,r.bar=r.bar)
+print(result,digits=digits)
+invisible(result)
+}
+
